import _queue
import logging
import select
from socket import *
from threading import Thread
from multiprocessing import Queue
from pynanopix.MultiprocessLogger import MultiprocessLogger, LoggerMessage

class UdpConnectionManager:

    IN_UDP_PORT = 14550
    SOCKET_READY_TIMEOUT = 0.01

    def __init__(self):
        self._rx_udp_socket = socket(AF_INET, SOCK_DGRAM)
        self._tx_udp_socket = None
        self._rx_udp_socket.bind(('', UdpConnectionManager.IN_UDP_PORT))
        self._nanopix_ip = None
        self._nanopix_port = None
        self._is_running = False
        self._conn_thread = Thread(target=self._main_routine)
        self._tx_queue = Queue()
        self._rx_queue = Queue()
        self._log_queue = MultiprocessLogger.get_queue()

    def tx_data(self, bytes_data):
        self._tx_queue.put(bytes_data, False)

    def get_rx_queue(self) -> Queue:
        return self._rx_queue

    def start(self):
        self._nanopix_ip = None
        self._nanopix_port = None
        self._is_running = True
        self._tx_udp_socket = None
        self._conn_thread.start()

    def stop(self):
        self._is_running = False
        self._conn_thread.join()

    def _main_routine(self):
        self._log_queue.put(LoggerMessage("ConnectionManager main routine started", logging.INFO))
        self._rx_udp_socket.setblocking(False)

        while True:
            if self._is_running:
                ready = select.select([self._rx_udp_socket], [], [], UdpConnectionManager.SOCKET_READY_TIMEOUT)
                if ready[0]:
                    msg, address = self._rx_udp_socket.recvfrom(256)

                    self._rx_queue.put(msg, False)

                    if self._nanopix_ip == None:
                        self._nanopix_ip = address[0]
                        self._nanopix_port = address[1]
                        self._tx_udp_socket = socket(AF_INET, SOCK_DGRAM)
                        self._tx_udp_socket.connect((self._nanopix_ip, self._nanopix_port))
                        self._log_queue.put(LoggerMessage(
                            "Nanopix binded: %s:%d" % (self._nanopix_ip, self._nanopix_port), logging.INFO))

            while self._tx_queue.qsize() > 0:
                data = None
                try:
                    data = self._tx_queue.get(False)
                except _queue.Empty:
                    break
                if data is not None:
                    self._tx_udp_socket.send(bytearray(data))

            if not self._is_running and self._tx_queue.qsize() == 0 and self._rx_queue.qsize() == 0:
                self._rx_udp_socket.close()
                if self._tx_udp_socket is not None:
                    self._tx_udp_socket.close()
                break

        self._log_queue.put(LoggerMessage("ConnectionManager main routine finished", logging.INFO))


