import logging

from logging import handlers
from multiprocessing import Queue
from queue import Empty
from threading import Thread

class LoggerMessage:
    def __init__(self, msg: str, level):
        self.msg = msg
        self.level = level

class MultiprocessLogger:
    _msg_queue = Queue()
    def __init__(self, filename: str, level, interval_hours: int, backup_count: int):
        self._is_running = False
        self._logger = logging.getLogger(filename)
        self._level = level
        self._setup_logger(filename, interval_hours, backup_count)
        self._thread = Thread(target=self._logger_routine)

    def start(self):
        self._is_running = True
        self._thread.start()

    def stop(self):
        self._is_running = False

    def join(self):
        self._thread.join()

    @staticmethod
    def get_queue():
        return MultiprocessLogger._msg_queue

    def _setup_logger(self, filename, interval_hours, backup_count):
        self._logger.setLevel(self._level)

        formatter = logging.Formatter(u"%(levelname)s - %(asctime)s - %(name)s - %(message)s")
        logHandler = handlers.TimedRotatingFileHandler(filename, when='H', interval=interval_hours, backupCount=backup_count)
        logHandler.setFormatter(formatter)

        self._logger.addHandler(logHandler)

    def _logger_routine(self):
        while True:
            try:
                msg = MultiprocessLogger._msg_queue.get(block=True, timeout=1)
                self._logger.log(msg.level, msg.msg)
            except Empty as e:
                pass
            except Exception as e:
                self._logger.log(logging.FATAL, "MultiprocessLogger EXCEPTION: " + str(e))

            if (MultiprocessLogger._msg_queue.qsize() == 0) and not self._is_running:
                break

        self._logger.log(logging.INFO, "MultiprocessLogger stopped")